@@ -55,16 +55,11 @@
 //
 //----------------------------------------------------------------------------
 
-#ifdef WIN32
-    #include <GL/glew.h>
-#else
-	#define GL_GLEXT_PROTOTYPES
-#endif
-
 #include <playExr.h>
 #include <ctlToLut.h>
 #include <fileNameForFrame.h>
 #include <FileReadingThread.h>
+#include <osDependent.h>
 #include <ImageBuffers.h>
 #include <Timer.h>
 #include <ImfThreading.h>
@@ -75,9 +70,6 @@
 #include <ImfArray.h>
 #include <half.h>
 #include <stdlib.h>
-#include <GL/glut.h>
-#include <GL/gl.h>
-#include <Cg/cgGL.h>
 #include <sstream>
 #include <iomanip>
 #include <cmath>
@@ -133,6 +125,16 @@ bool enableCtl = true;		// If enableCtl is true, CTL transforms
 				// has been applied.  If enableCtl is false,
 				// no CTL transforms are applied.
 
+const size_t lutN = 64;		// The 3D color lookup table that is used
+				// to approximate the CTL transforms has
+				// lutN by lutN by lutN entries.
+
+bool hwTexInterpolation = true;	// Flag that controls whether the Cg shader
+				// that performs 3D color table lookups
+				// relies on hardware-interpolated texture
+				// lookups or if the shader itself interpolates
+				// between texture samples
+
 bool showTextOverlay = true;	// Flag that controls whether the actual
 				// frame rate and the current exposure
 				// setting are displayed
@@ -228,7 +230,7 @@ initializeImageBuffers
 	//
 	// The pixel buffers for the tree image channels (RGB)
 	// are padded with a fourth dummy channel (A) and interleaved
-	// (RGBARGBARGBA...).  All three buffers have ﬆhe same width
+	// (RGBARGBARGBA...).  All three buffers have the same width
 	// and height as the frame.
 	//
 
@@ -287,7 +289,8 @@ void
 computeWindowSizes
     (Box2i &dataWindow,
      Box2i &displayWindow,
-     float pixelAspectRatio)
+     float pixelAspectRatio,
+     float xyScale)
 {
     //
     // Beginning with the data and display window of the first frame
@@ -341,7 +344,7 @@ computeWindowSizes
     //
     // The size and location of the image within the OpenGL window
     // is determined by the (possibly stretched) data window.
-    // The data window must be transformed from OpnEXR pixel space
+    // The data window must be transformed from OpenEXR pixel space
     // to OpenGL coordinates (with y going from bottom to top).
     //
 
@@ -350,6 +353,19 @@ computeWindowSizes
 
     drawRect.max.x = dataWindow.max.x + 1;
     drawRect.max.y = displayWindow.max.y - dataWindow.min.y + 1;
+
+    //
+    // The user may have requested that the images be
+    // displayed smaller or larger than their original size.
+    //
+
+    glWindowWidth  = int (floor (glWindowWidth  * xyScale + 0.5));
+    glWindowHeight = int (floor (glWindowHeight * xyScale + 0.5));
+
+    drawRect.min.x = int (floor (drawRect.min.x * xyScale + 0.5));
+    drawRect.min.y = int (floor (drawRect.min.y * xyScale + 0.5));
+    drawRect.max.x = int (floor (drawRect.max.x * xyScale + 0.5));
+    drawRect.max.y = int (floor (drawRect.max.y * xyScale + 0.5));
 }
 
 
@@ -362,7 +378,7 @@ computeWindowSizes
 
 CGcontext cgContext;
 CGprogram cgProgram;
-CGprofile cgProfile = CG_PROFILE_FP30;
+CGprofile cgProfile;
 
 
 void
@@ -389,10 +405,10 @@ const char shaderLuminanceChromaSource[] =
 "                                                                          \n"
 "    Out                                                                   \n"
 "    main (float2 tc: TEXCOORD0,                                           \n"
-"          uniform sampler yImage: TEXUNIT0,                               \n"
-"          uniform sampler ryImage: TEXUNIT1,                              \n"
-"          uniform sampler byImage: TEXUNIT2,                              \n"
-"          uniform sampler lut: TEXUNIT3,                                  \n"
+"          uniform sampler2D yImage: TEXUNIT0,                             \n"
+"          uniform sampler2D ryImage: TEXUNIT1,                            \n"
+"          uniform sampler2D byImage: TEXUNIT2,                            \n"
+"          uniform sampler3D lut: TEXUNIT3,                                \n"
 "          uniform float3 yw,                                              \n"
 "          uniform float expMult,                                          \n"
 "          uniform float videoGamma,                                       \n"
@@ -400,6 +416,7 @@ const char shaderLuminanceChromaSource[] =
 "          uniform float lutMax,                                           \n"
 "          uniform float lutM,                                             \n"
 "          uniform float lutT,                                             \n"
+"          uniform float lutF,                                             \n"
 "          uniform float enableLut)                                        \n"
 "    {                                                                     \n"
 "        //                                                                \n"
@@ -425,9 +442,50 @@ const char shaderLuminanceChromaSource[] =
 "        //                                                                \n"
 "                                                                          \n"
 "        if (enableLut)                                                    \n"
-"        {                                                                 \n"
-"            color = lutT + lutM * log (clamp (color, lutMin, lutMax));    \n"
-"            color = exp (tex3D (lut, color).rgb);                         \n"
+"	 {                                                                 \n"
+"            if (lutF)                                                     \n"
+"            {                                                             \n"
+"                //                                                        \n"
+"                // Texture hardware does not support                      \n"
+"                // interpolation between texture samples.                 \n"
+"                //                                                        \n"
+"                                                                          \n"
+"                half3 i = lutF * half3                                    \n"
+"                    (lutT + lutM * log (clamp (color, lutMin, lutMax)));  \n"
+"                                                                          \n"
+"                half3 fi = floor (i);                                     \n"
+"                half3 fj = fi + 1;                                        \n"
+"                half3 s = i - fi;                                         \n"
+"                                                                          \n"
+"                fi = fi / lutF;                                           \n"
+"                fj = fj / lutF;                                           \n"
+"                                                                          \n"
+"                half3 c0 = tex3D (lut, half3 (fi.x, fi.y, fi.z)).rgb;     \n"
+"                half3 c1 = tex3D (lut, half3 (fj.x, fi.y, fi.z)).rgb;     \n"
+"                half3 c2 = tex3D (lut, half3 (fi.x, fj.y, fi.z)).rgb;     \n"
+"                half3 c3 = tex3D (lut, half3 (fj.x, fj.y, fi.z)).rgb;     \n"
+"                half3 c4 = tex3D (lut, half3 (fi.x, fi.y, fi.z)).rgb;     \n"
+"                half3 c5 = tex3D (lut, half3 (fj.x, fi.y, fj.z)).rgb;     \n"
+"                half3 c6 = tex3D (lut, half3 (fi.x, fj.y, fj.z)).rgb;     \n"
+"                half3 c7 = tex3D (lut, half3 (fj.x, fj.y, fj.z)).rgb;     \n"
+"                                                                          \n"
+"                color = ((c0 * (1-s.x) + c1 * s.x) * (1-s.y) +            \n"
+"                         (c2 * (1-s.x) + c3 * s.x) *  s.y) * (1-s.z) +    \n"
+"                        ((c4 * (1-s.x) + c5 * s.x) * (1-s.y) +            \n"
+"                         (c6 * (1-s.x) + c7 * s.x) *  s.y) * s.z;         \n"
+"                                                                          \n"
+"                color = exp (color);                                      \n"
+"            }                                                             \n"
+"            else                                                          \n"
+"            {                                                             \n"
+"                //                                                        \n"
+"                // Texture hardware supports trilinear                    \n"
+"                // interpolation between texture samples.                 \n"
+"                //                                                        \n"
+"                                                                          \n"
+"                color = lutT + lutM * log (clamp (color, lutMin, lutMax));\n"
+"                color = exp (tex3D (lut, color).rgb);                     \n"
+"            }                                                             \n"
 "        }                                                                 \n"
 "                                                                          \n"
 "        //                                                                \n"
@@ -451,6 +509,7 @@ initShaderLuminanceChroma
     cgSetErrorCallback (handleCgErrors);
 
     cgContext = cgCreateContext();
+    cgProfile = cgGLGetLatestProfile (CG_GL_FRAGMENT);
     cgGLSetOptimalOptions (cgProfile);
 
     cgProgram =
@@ -459,6 +518,7 @@ initShaderLuminanceChroma
 
     cgGLLoadProgram (cgProgram);
     cgGLBindProgram (cgProgram);
+    
     cgGLEnableProfile (cgProfile);
 
     CGparameter ywParam = cgGetNamedParameter (cgProgram, "yw");
@@ -484,6 +544,9 @@ initShaderLuminanceChroma
 
     CGparameter enableLutParam = cgGetNamedParameter (cgProgram, "enableLut");
     cgSetParameter1f (enableLutParam, enableCtl? 1.0: 0.0);
+
+    CGparameter lutFParam = cgGetNamedParameter (cgProgram, "lutF");
+    cgSetParameter1f (lutFParam, hwTexInterpolation? 0: lutN - 1);
 }
 
 
@@ -500,14 +563,15 @@ const char shaderRgbSource[] =
 "                                                                          \n"
 "    Out                                                                   \n"
 "    main (float2 tc: TEXCOORD0,                                           \n"
-"          uniform sampler rgbImage: TEXUNIT0,                             \n"
-"          uniform sampler lut: TEXUNIT3,                                  \n"
+"          uniform sampler2D rgbImage: TEXUNIT0,                           \n"
+"          uniform sampler3D lut: TEXUNIT3,                                \n"
 "          uniform float expMult,                                          \n"
 "          uniform float videoGamma,                                       \n"
 "          uniform float lutMin,                                           \n"
 "          uniform float lutMax,                                           \n"
 "          uniform float lutM,                                             \n"
 "          uniform float lutT,                                             \n"
+"          uniform float lutF,                                             \n"
 "          uniform float enableLut)                                        \n"
 "    {                                                                     \n"
 "        //                                                                \n"
@@ -521,9 +585,50 @@ const char shaderRgbSource[] =
 "        //                                                                \n"
 "                                                                          \n"
 "        if (enableLut)                                                    \n"
-"        {                                                                 \n"
-"            color = lutT + lutM * log (clamp (color, lutMin, lutMax));    \n"
-"            color = exp (tex3D (lut, color).rgb);                         \n"
+"	 {                                                                 \n"
+"            if (lutF)                                                     \n"
+"            {                                                             \n"
+"                //                                                        \n"
+"                // Texture hardware does not support                      \n"
+"                // interpolation between texture samples.                 \n"
+"                //                                                        \n"
+"                                                                          \n"
+"                half3 i = lutF * half3                                    \n"
+"                    (lutT + lutM * log (clamp (color, lutMin, lutMax)));  \n"
+"                                                                          \n"
+"                half3 fi = floor (i);                                     \n"
+"                half3 fj = fi + 1;                                        \n"
+"                half3 s = i - fi;                                         \n"
+"                                                                          \n"
+"                fi = fi / lutF;                                           \n"
+"                fj = fj / lutF;                                           \n"
+"                                                                          \n"
+"                half3 c0 = tex3D (lut, half3 (fi.x, fi.y, fi.z)).rgb;     \n"
+"                half3 c1 = tex3D (lut, half3 (fj.x, fi.y, fi.z)).rgb;     \n"
+"                half3 c2 = tex3D (lut, half3 (fi.x, fj.y, fi.z)).rgb;     \n"
+"                half3 c3 = tex3D (lut, half3 (fj.x, fj.y, fi.z)).rgb;     \n"
+"                half3 c4 = tex3D (lut, half3 (fi.x, fi.y, fi.z)).rgb;     \n"
+"                half3 c5 = tex3D (lut, half3 (fj.x, fi.y, fj.z)).rgb;     \n"
+"                half3 c6 = tex3D (lut, half3 (fi.x, fj.y, fj.z)).rgb;     \n"
+"                half3 c7 = tex3D (lut, half3 (fj.x, fj.y, fj.z)).rgb;     \n"
+"                                                                          \n"
+"                color = ((c0 * (1-s.x) + c1 * s.x) * (1-s.y) +            \n"
+"                         (c2 * (1-s.x) + c3 * s.x) *  s.y) * (1-s.z) +    \n"
+"                        ((c4 * (1-s.x) + c5 * s.x) * (1-s.y) +            \n"
+"                         (c6 * (1-s.x) + c7 * s.x) *  s.y) * s.z;         \n"
+"                                                                          \n"
+"                color = exp (color);                                      \n"
+"            }                                                             \n"
+"            else                                                          \n"
+"            {                                                             \n"
+"                //                                                        \n"
+"                // Texture hardware supports trilinear                    \n"
+"                // interpolation between texture samples.                 \n"
+"                //                                                        \n"
+"                                                                          \n"
+"                color = lutT + lutM * log (clamp (color, lutMin, lutMax));\n"
+"                color = exp (tex3D (lut, color).rgb);                     \n"
+"            }                                                             \n"
 "        }                                                                 \n"
 "                                                                          \n"
 "        //                                                                \n"
@@ -547,6 +652,7 @@ initShaderRgb
     cgSetErrorCallback (handleCgErrors);
 
     cgContext = cgCreateContext();
+    cgProfile = cgGLGetLatestProfile (CG_GL_FRAGMENT);
     cgGLSetOptimalOptions (cgProfile);
 
     cgProgram =
@@ -577,6 +683,9 @@ initShaderRgb
 
     CGparameter enableLutParam = cgGetNamedParameter (cgProgram, "enableLut");
     cgSetParameter1f (enableLutParam, enableCtl? 1.0: 0.0);
+
+    CGparameter lutFParam = cgGetNamedParameter (cgProgram, "lutF");
+    cgSetParameter1f (lutFParam, hwTexInterpolation? 0: lutN - 1);
 }
 
 
@@ -717,7 +826,7 @@ init3DLut
     //
     // The 3D lookup table covers a range from lutMin to lutMax or
     // NUM_STOPS f-stops above and below 0.18 or MIDDLE_GRAY.  The
-    // size of the table is N by N by N samples.
+    // size of the table is lutN by lutN by lutN samples.
     //
     // In order make the distribution of the samples in the table
     // approximately perceptually uniform, the Cg shaders that use
@@ -750,8 +859,7 @@ init3DLut
     lutM = 1 / (logLutMax - logLutMin);
     lutT = -lutM * logLutMin;
 
-    static const size_t N = 64;
-    size_t LUT_SIZE = N * N * N * 4;
+    size_t LUT_SIZE = lutN * lutN * lutN * 4;
 
     //
     // Build a 3D array of RGB input pixel values.
@@ -760,22 +868,22 @@ init3DLut
 
     Array<half> pixelValues (LUT_SIZE);
 
-    for (size_t ib = 0; ib < N; ++ib)
+    for (size_t ib = 0; ib < lutN; ++ib)
     {
-	float b = ib / (N - 1.0);
+	float b = ib / (lutN - 1.0);
 	half B = exp ((b - lutT) / lutM);
 
-	for (size_t ig = 0; ig < N; ++ig)
+	for (size_t ig = 0; ig < lutN; ++ig)
 	{
-	    float g = ig / (N - 1.0);
+	    float g = ig / (lutN - 1.0);
 	    half G = exp ((g - lutT) / lutM);
 
-	    for (int ir = 0; ir < N; ++ir)
+	    for (int ir = 0; ir < lutN; ++ir)
 	    {
-		float r = ir / (N - 1.0);
+		float r = ir / (lutN - 1.0);
 		half R = exp ((r - lutT) / lutM);
 
-		size_t i = (ib * N * N + ig * N + ir) * 4;
+		size_t i = (ib * lutN * lutN + ig * lutN + ir) * 4;
 		pixelValues[i + 0] = R;
 		pixelValues[i + 1] = G;
 		pixelValues[i + 2] = B;
@@ -847,7 +955,7 @@ init3DLut
     glTexImage3D (GL_TEXTURE_3D,
 		  0,			// level
 		  GL_RGBA16F_ARB,	// internalFormat
-		  N, N, N,		// width, height, depth
+		  lutN, lutN, lutN,	// width, height, depth
 		  0,			// border
 		  GL_RGBA,		// format
 		  GL_HALF_FLOAT_ARB,	// type
@@ -894,7 +1002,7 @@ drawStringWithBorder (GLfloat x, GLfloat y, const char str[])
     drawString (x + 1, y + 1, str);
     glLineWidth (2);
     glColor4f (0.8, 0.8, 0.8, 1);
-    drawString (y, y, str);
+    drawString (x, y, str);
     glDisable (GL_LINE_SMOOTH);
     glDisable (GL_BLEND);
 }
@@ -1292,7 +1400,9 @@ playExr (const char fileNameTemplate[],
 	 int lastFrame,
 	 int numThreads,
 	 float fps,
-	 const vector<string> &transformNames)
+	 float xyScale,
+	 const vector<string> &transformNames,
+	 bool useHwTexInterpolation)
 {
     //
     // Set the number of threads the IlmImf library
@@ -1331,7 +1441,8 @@ playExr (const char fileNameTemplate[],
 
     computeWindowSizes (header.dataWindow(),
 			header.displayWindow(),
-			header.pixelAspectRatio());
+			header.pixelAspectRatio(),
+			xyScale);
 
     //
     // Create an OpenGL window
@@ -1345,36 +1456,10 @@ playExr (const char fileNameTemplate[],
     glutDisplayFunc (redrawWindow);
 
     //
-    // Verify that OpenGL supports the extensions we need.
+    // Verify that OpenGL supports the extensions we need
     //
 
-    if (!glutExtensionSupported ("GL_ARB_texture_float") ||
-	!glutExtensionSupported ("GL_ARB_half_float_pixel"))
-    {
-	cerr << "This program requires OpenGL support for "
-		"16-bit floating-point textures." << endl;
-	exit (1);
-    }
-
-    if (!glutExtensionSupported ("GL_ARB_fragment_shader"))
-    if (!glutExtensionSupported ("GL_EXT_Cg_shader"))
-    {
-	cerr << "This program requires OpenGL support for "
-		"fragment shaders and the Cg shading language." << endl;
-	exit (1);
-    }
-
-#ifdef WIN32
-
-    GLenum err = glewInit();
-
-    if (GLEW_OK != err)
-    {
-	cerr << "Cannot initialize glew: " << glewGetErrorString (err) << endl;
-	exit (1);
-    }
-
-#endif
+    initAndCheckGlExtensions();
 
     //
     // Initialize textures and Cg shaders
@@ -1387,6 +1472,8 @@ playExr (const char fileNameTemplate[],
 
     init3DLut (transformNames, header, xMin, xMax, m, t);
 
+    hwTexInterpolation = useHwTexInterpolation;
+
     if (ib.rgbMode)
     {
 	initShaderRgb (xMin, xMax, m, t);
